@@ -33,7 +33,7 @@ our %LOCAL_CONFIG = (
 	MONGO_PORT     => 27017,
 	SQLNAME        => 'hockey',
 	SQLUSER        => 'root',
-	BASE_DIR       => '/hockey',
+	BASE_DIR       => '/tmp/hockey',
 	MERGED_FILE    => 'merged.storable',
 	NORMALIZED_FILE => 'normalized.storable',
 	NORMALIZED_JSON => 'normalized.json',
